/*
 *  Copyright 2013-2050 Emmanuel BRUN (contact@amapj.fr)
 * 
 *  This file is part of AmapJ.
 *  
 *  AmapJ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  AmapJ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with AmapJ.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 */
 package fr.amapj.view.views.saisiecontrat.table;

import java.text.SimpleDateFormat;
import java.util.List;

import fr.amapj.common.AmapjRuntimeException;
import fr.amapj.common.FormatUtils;
import fr.amapj.service.services.mescontrats.ContratDTO;
import fr.amapj.service.services.stockservice.verifstock.VerifStockService;
import fr.amapj.view.engine.grid.integergrid.IntegerGridCell;
import fr.amapj.view.engine.grid.integertable.IntegerTableParam;
import fr.amapj.view.engine.grid.integertable.PopupIntegerTable;
import fr.amapj.view.engine.popup.messagepopup.MessagePopup;
import fr.amapj.view.engine.tools.BaseUiTools;
import fr.amapj.view.views.saisiecontrat.SaisieContrat.ModeSaisie;
import fr.amapj.view.views.saisiecontrat.SaisieContrat.SaisieContratData;
import fr.amapj.view.views.saisiecontrat.reglesaisie.PopupRegleSaisieVisuErreur;

/**
 * Popup pour la saisie des quantites sur une seule date, en mode table 
 *  
 */
public class PopupSaisieQteTable extends PopupIntegerTable
{	
	SimpleDateFormat df = new SimpleDateFormat("dd/MM/yyyy");
	
	private ContratDTO contratDTO;
	
	private SaisieContratData data;

	private int indexLig;
	
	/**
	 * 
	 *  
	 */
	public PopupSaisieQteTable(SaisieContratData data)
	{
		super();
		this.data = data;
		this.contratDTO = data.contratDTO;
		
		
		//
		popupTitle = "Mon contrat "+contratDTO.nom;
		
		// 
		param.readOnly = (data.modeSaisie==ModeSaisie.READ_ONLY);
		param.messageSpecifique = s(data.messageSpecifique);
		param.libButtonSave = "Continuer ...";
		
	}
	
	
	
	public void loadParam()
	{		
		//
		if (contratDTO.contratLigs.size()!=1)
		{
			throw new AmapjRuntimeException();
		}
		this.indexLig = 0;
		if (param.readOnly==false && contratDTO.contratLigs.get(indexLig).isModifiable==false)
		{
			throw new AmapjRuntimeException();
		}
		
		//
		param.nbCell = contratDTO.contratColumns.size();
		
		//
		IntegerGridCell[][] cs = contratDTO.extractCells();
		param.cell = contratDTO.convertOneLineOfCells(cs, indexLig);

		//
		param.allowedEmpty = false;
		param.libPrixTotal = "Montant pour la date du "+FormatUtils.getStdDate().format(contratDTO.contratLigs.get(indexLig).date);
		
		// Largeur des colonnes
		computeWidthAndHeigth(param);
		
		/*
		param.nbCellPerCol = 3; // TODO : à calculer en automatique 
		param.cellWidth = 250;
		param.cellHeight = 150; // TODO : à calculer en automatique
		*/ 
		
	}


	/**
	 * Calcul de la largeur et de la hauteur des cellules, en fonction du texte présent à l'intérieur
	 * @param param
	 */
	private void computeWidthAndHeigth(IntegerTableParam param) 
	{
		param.cellHeight = 150;
		
		int width = BaseUiTools.getWidth();
		if (width<600)
		{
			param.nbCellPerCol = 1;
			param.cellWidth = width-140;
		}
		else if (width<900)
		{
			param.nbCellPerCol = 2;
			param.cellWidth = 260;
		}
		else if (width<1200)
		{
			param.nbCellPerCol = 3;
			param.cellWidth = 260;
		}
		else 
		{
			param.nbCellPerCol = 4;
			param.cellWidth = 260;
			
			// Traitement de certains cas particuliers
			if (param.nbCell==6 || param.nbCell==9)
			{
				param.nbCellPerCol = 3;
				param.cellWidth = 260;
			}
			
		}
	}



	@Override
	protected void handleContinuer()
	{
		data.validate();
		close();
	}

	@Override
	public boolean performSauvegarder()
	{
		// On recopie les données dans le contratDto
		for (int j = 0; j < param.nbCell; j++)
		{
			contratDTO.cell[indexLig][j].qte = param.cell[j].qte;
		}
		
		// On verifie les regles de saisies
		if (PopupRegleSaisieVisuErreur.performCheck(data)==false)
		{
			return false;
		}
		
		// On verifie le stock
		data.verifStockDTO.setQteMe(contratDTO);
		if (data.verifStockDTO.isStockSuffisant()==false)
		{
			List<String> ls = new VerifStockService().computePrettyMessage(data.verifStockDTO);
			new MessagePopup("Impossible de continuer",ls).open();
			return false;
		}
		
		
		// 
		data.validate();
		
		return true;
	}	
}
